@@ -148,6 +148,7 @@ static char *fifopath = NULL;
 static int fifo_fd = 0;
 static char *pidfile = NULL;
 static char *svcname = NULL;
+static bool verbose = false;
 
 extern char **environ;
 
@@ -420,8 +421,8 @@ static void child_process(char *exec, char **argv)
 
 	TAILQ_FOREACH(env, env_list, entries) {
 		if ((strncmp(env->value, "RC_", 3) == 0 &&
-			strncmp(env->value, "RC_SERVICE=", 10) != 0 &&
-			strncmp(env->value, "RC_SVCNAME=", 10) != 0) ||
+			strncmp(env->value, "RC_SERVICE=", 11) != 0 &&
+			strncmp(env->value, "RC_SVCNAME=", 11) != 0) ||
 		    strncmp(env->value, "SSD_NICELEVEL=", 14) == 0)
 		{
 			p = strchr(env->value, '=');
@@ -569,7 +570,8 @@ static void supervisor(char *exec, char **argv)
 				buf[count] = 0;
 			if (count == 0)
 				continue;
-			syslog(LOG_DEBUG, "Received %s from fifo", buf);
+			if (verbose)
+				syslog(LOG_DEBUG, "Received %s from fifo", buf);
 			if (strncasecmp(buf, "sig", 3) == 0) {
 				if ((sscanf(buf, "%s %d", cmd, &sig_send) == 2)
 						&& (sig_send >= 0 && sig_send < NSIG)) {
@@ -585,7 +587,8 @@ static void supervisor(char *exec, char **argv)
 		if (do_healthcheck) {
 			do_healthcheck = 0;
 			alarm(0);
-			syslog(LOG_DEBUG, "running health check for %s", svcname);
+			if (verbose)
+				syslog(LOG_DEBUG, "running health check for %s", svcname);
 			health_pid = exec_command("healthcheck");
 			health_status = rc_waitpid(health_pid);
 			if (WIFEXITED(health_status) && WEXITSTATUS(health_status) == 0)
@@ -645,6 +648,8 @@ static void supervisor(char *exec, char **argv)
 			ts.tv_sec = respawn_delay;
 			ts.tv_nsec = 0;
 			nanosleep(&ts, NULL);
+			if (exiting)
+				continue;
 			child_pid = fork();
 			if (child_pid == -1) {
 				syslog(LOG_ERR, "%s: fork: %s", applet, strerror(errno));
@@ -835,7 +840,7 @@ int main(int argc, char **argv)
 
 		case 'm':  /* --respawn-max count */
 			n = sscanf(optarg, "%d", &respawn_max);
-			if (n	!= 1 || respawn_max < 1)
+			if (n	!= 1 || respawn_max < 0)
 				eerrorx("Invalid respawn-max value '%s'", optarg);
 			break;
 
@@ -852,10 +857,11 @@ int main(int argc, char **argv)
 
 		case 'u':  /* --user <username>|<uid> */
 		{
+			char dummy[2];
 			p = optarg;
 			tmp = strsep(&p, ":");
 			changeuser = xstrdup(tmp);
-			if (sscanf(tmp, "%d", &tid) != 1)
+			if (sscanf(tmp, "%d%1s", &tid, dummy) != 1)
 				pw = getpwnam(tmp);
 			else
 				pw = getpwuid((uid_t)tid);
@@ -876,7 +882,7 @@ int main(int argc, char **argv)
 
 			if (p) {
 				tmp = strsep (&p, ":");
-				if (sscanf(tmp, "%d", &tid) != 1)
+				if (sscanf(tmp, "%d%1s", &tid, dummy) != 1)
 					gr = getgrnam(tmp);
 				else
 					gr = getgrgid((gid_t) tid);
@@ -904,6 +910,7 @@ int main(int argc, char **argv)
 		case_RC_COMMON_GETOPT
 		}
 
+	verbose = rc_yesno(getenv ("EINFO_VERBOSE"));
 	endpwent();
 	argc -= optind;
 	argv += optind;
@@ -983,7 +990,7 @@ int main(int argc, char **argv)
 			}
 			if (!exists(exec_file)) {
 				eerror("%s: %s does not exist", applet,
-				    *exec_file ? exec_file : exec);
+				    exec_file ? exec_file : exec);
 				free(exec_file);
 				exit(EXIT_FAILURE);
 			}
